from ural.types import AnyUrlTarget

def fingerprint_url(url: str, unsplit: bool = ..., strip_suffix: bool = ...) -> str: ...
def fingerprint_hostname(hostname: str, strip_suffix: bool = ...) -> str: ...
def get_fingerprinted_hostname(
    url: AnyUrlTarget, infer_redirection: bool = True, strip_suffix: bool = ...
) -> str: ...
