from typing import Optional, Tuple
from ural.types import AnyUrlTarget

def upgrade(transient: bool = ...) -> None: ...
def get_domain_name(url: AnyUrlTarget) -> Optional[str]: ...
def has_valid_suffix(url: AnyUrlTarget) -> bool: ...
def split_suffix(url: AnyUrlTarget) -> Optional[Tuple[str, str]]: ...
def is_valid_tld(tld: str) -> bool: ...
def has_valid_tld(url: AnyUrlTarget) -> bool: ...
